part of edit_screen;

class EditorController {
  final List<EditorItem> editorItems;
  final Map<String, Object> dataEntity = {};
  final String onSaveProcedure;
  String _messageInvalidDialog;
  bool _isDirtyData = false;
  RxBool _isConfirm = false.obs;
  RxBool _isWaiting = false.obs;
  RxBool _succeed = RxBool(false);
  RxBool _isShowInValidDialog = false.obs;
  VoidCallback onSuccess;
  static Map<String, Object> mapDataReturn;

  EditorController(
    Map<String, Object> dataEntity,
    this.onSaveProcedure,
    this.editorItems,
  ) {
    _addDefaultValues();
    initalValue(dataEntity);
  }

  void initalValue(Map<String, Object> initialMapValue) {
    if (initialMapValue != null) {
      dataEntity.addEntries(initialMapValue.entries);
    }
  }

  void onValueChanged(String fieldName, Object value) {
    Object _value = value;
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem?.type == EEditorType.int && (value is String)) {
      _value = int.tryParse(value);
    } else if (editorItem?.type == EEditorType.double && (value is String)) {
      _value = double.tryParse(value);
    } else if (editorItem?.type == EEditorType.money && (value is String)) {
      _value = double.tryParse(value.replaceAll(RegExp(','), ''));
    }

    dataEntity.update(
      fieldName,
      (_) => _value,
      ifAbsent: () => _value,
    );
    _isDirtyData = true;
  }

  void confirmAndSave({VoidCallback onSuccess}) {
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    _isConfirm.value = true;
  }

  void showInvalidDialog({String message}) {
    _messageInvalidDialog = message;
    _isShowInValidDialog.value = true;
  }

  String getHintText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem?.hint != null && editorItem.hint.isNotEmpty) {
      return editorItem.hint;
    }
    if (editorItem?.type == EEditorType.money) {
      return Message.getMessage('EditorWidget_nhap_so_tien');
    }
    if (editorItem?.title != null && editorItem.title.isNotEmpty) {
      return Message.getMessage('EditorWidget_nhap') +
          ' ' +
          _removeLastStarSymbolAndLowCase(editorItem.title);
    }
    return Message.getMessage('EditorWidget_nhap_thong_tin');
  }

  String getInvalidText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem?.invalidMessage != null &&
        editorItem.invalidMessage.isNotEmpty) {
      return editorItem.invalidMessage;
    }
    return _getEmptyMessageValidator(editorItem);
  }

  String getText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);

    if (editorItem == null || dataEntity[fieldName] == null) return '';

    if (editorItem.type == EEditorType.money) {
      return PriceInputFormatter.format(
          (dataEntity[fieldName] as double).toInt().toString());
    }

    return dataEntity[fieldName].toString();
  }

  String getValidator(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    String _checkMinMax = checkMinMax(editorItem);
    if (_checkMinMax != null) return _checkMinMax;

    bool isRequired = editorItem?.isRequired ?? false;

    String title = editorItem?.title ?? '';
    if (title.isNotEmpty && !isRequired) {
      isRequired = title.startsWith('*') || title.endsWith('*');
    }

    if (!isRequired) return null;

    if (editorItem.validator != null && dataEntity[fieldName] != null) {
      return editorItem.validator(dataEntity[fieldName]);
    }

    switch (editorItem.type) {
      case EEditorType.image:
        return _validateImage(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.string:
        return _validateString(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.phone:
        return Methods.validPhoneNumber(dataEntity[fieldName] ?? '')
            ? null
            : _getInvalidMessageValidator(
                Message.getMessage('EditorWidget_so_dien_thoai'));
      case EEditorType.email:
        return (dataEntity[fieldName] as String).isEmail
            ? null
            : _getInvalidEmailValidator(editorItem);
      case EEditorType.int:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.double:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.money:
        return _validateMoney(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
        break;
      case EEditorType.datetime:
        return _validateDateTime(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      default:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
    }
  }

  TextInputType getTextInputType(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem == null) return null;
    switch (editorItem.type) {
      case EEditorType.int:
      case EEditorType.money:
        return TextInputType.number;
      case EEditorType.double:
        return TextInputType.numberWithOptions(decimal: true);
      case EEditorType.phone:
        return TextInputType.phone;
      case EEditorType.email:
        return TextInputType.emailAddress;
      case EEditorType.datetime:
        return TextInputType.datetime;
      case EEditorType.name:
      case EEditorType.string:
      case EEditorType.image:
      default:
        return null;
    }
  }

  List<TextInputFormatter> getInputFormatters(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);

    if (editorItem == null) return <TextInputFormatter>[];

    if (editorItem.type == EEditorType.money) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        PriceInputFormatter(),
      ];
    }

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.phone) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
      ];
    }

    if (editorItem.type == EEditorType.name) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.allow(
          RegExp(
              '[a-zA-Z_ÀÁÂÃÈÉÊÌÍÒÓÔÕÙÚĂĐĨŨƠàáâãèéêìíòóôõùúăđĩũơƯĂẠẢẤẦẨẪẬẮẰẲẴẶẸẺẼỀỀỂưăạảấầẩẫậắằẳẵặẹẻẽềềểỄỆỈỊỌỎỐỒỔỖỘỚỜỞỠỢỤỦỨỪễếệỉịọỏốồổỗộớờởỡợụủứừỬỮỰỲỴÝỶỸửữựỳỵỷỹ  \t]'),
        ),
      ];
    }

    return <TextInputFormatter>[];
  }

  String _getEmptyMessageValidator(EditorItem item) {
    if (item.invalidMessage != null && item.invalidMessage != "") {
      return item.invalidMessage;
    }
    // if (item.title != null && item.title != "") {
    //   return null;
    // }
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (item.title != null
            ? _removeLastStarSymbolAndLowCase(item.title)
            : Message.getMessage('EditorWidget_thong_tin'));
  }

  String _getInvalidMessageValidator(String title) {
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (title != null
            ? _removeLastStarSymbolAndLowCase(title)
            : Message.getMessage('EditorWidget_thong_tin')) +
        ' ' +
        Message.getMessage('EditorWidget_hop_le');
  }

  String _getInvalidEmailValidator(EditorItem item) {
    String email = Methods.getString(dataEntity, item.fieldName);
    if (email.isEmpty) {
      return item.invalidMessage;
    } else {
      return Message.getMessage('EDITOR_EMAIL_INVALID');
    }
  }

  String _removeLastStarSymbolAndLowCase(String str) {
    if (str != null) {
      return str.replaceAll('*', '').toLowerCase();
    }
    return '';
  }

  String getTitleByFieldName(String fieldName) {
    return getEditorItemByFieldName(fieldName)?.title;
  }

  bool _checkIsImagesKey(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem != null && editorItem.type == EEditorType.image) {
      return true;
    }
    return false;
  }

// #region Validate Data
  bool validate({int step, bool isShowInvalidDialog = true}) {
    if (!_checkValidData(step)) {
      print('validate');
      if (isShowInvalidDialog == true) {
        _isShowInValidDialog.value = true;
      }
      return false;
    } else {
      return true;
    }
  }

  bool _checkValidData(int step) {
    for (final editorItem in editorItems) {
      if ((step != null && editorItem.step != step) || !editorItem.isRequired) {
        continue;
      }
      if (getValidator(editorItem.fieldName) != null) {
        _messageInvalidDialog = _getEmptyMessageValidator(editorItem);
        return false;
      }
    }
    return true;
  }

  bool _validateString(dynamic value) {
    return (value is String) && value.isNotEmpty;
  }

  bool _validateMoney(dynamic value) {
    return (value is num && value > 0);
  }

  bool _validateDateTime(dynamic value) {
    return value is DateTime && value != null;
  }

  void _resetAfterDone() {
    _isWaiting.value = false;
    _succeed.value = null;
    _isDirtyData = false;
    _addDefaultValues();
  }

  void _resetAfterFail() {
    _isWaiting.value = false;
    _succeed.value = false;
    _isDirtyData = true;
  }

  void _addDefaultValues() {
    editorItems.forEach((item) {
      dataEntity.addAll({item.fieldName: item.defaultValue});
    });
  }

  EditorItem getEditorItemByFieldName(String fieldName) {
    final editorItem = editorItems.firstWhere(
        (element) => element.fieldName == fieldName,
        orElse: () => null);
    return editorItem;
  }

  bool _validateImage(List images) {
    if (images != null && images.isNotEmpty) {
      return true;
    }
    return false;
  }
}

extension MinMax on EditorController {
  String checkMinMax(EditorItem item) {
    if (item == null) return null;
    switch (item.type) {
      case EEditorType.image:
        if (dataEntity[item.fieldName] is Iterable) {
          int value = Methods.getList(dataEntity, item.fieldName).length;
          return _checkMinMaxInt(
            value,
            min: item.min?.toInt(),
            max: item.max?.toInt(),
            invalidMin: item.invalidMin ?? item.invalidMessage,
            invalidMax: item.invalidMax ?? item.invalidMessage,
          );
        }
        return null;
      case EEditorType.int:
        int value = Methods.getInt(dataEntity, item.fieldName);
        return _checkMinMaxInt(
          value,
          min: item.min?.toInt(),
          max: item.max?.toInt(),
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMax,
        );
      case EEditorType.double:
      case EEditorType.money:
        double value = Methods.getDouble(dataEntity, item.fieldName);
        return _checkMinMaxDouble(
          value,
          min: item.min,
          max: item.max,
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      case EEditorType.name:
      case EEditorType.string:
      case EEditorType.email:
      case EEditorType.phone:
        String text = Methods.getString(dataEntity, item.fieldName);

        return _checkMinMaxString(
          text,
          min: item.min?.toInt(),
          max: item.max?.toInt(),
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      default:
        return null;
    }
  }

  String _checkMinMaxString(
    String text, {
    int min,
    int max,
    String invalidMin,
    String invalidMax,
  }) {
    if (min != null && text.length < min) {
      return invalidMin;
    } else if (max != null && text.length > max) {
      return invalidMax;
    }
    return null;
  }

  String _checkMinMaxInt(
    int value, {
    int min,
    int max,
    String invalidMin,
    String invalidMax,
  }) {
    if (min != null && value < min) {
      return invalidMin;
    } else if (max != null && value > max) {
      return invalidMax;
    }
    return null;
  }

  String _checkMinMaxDouble(
    double value, {
    double min,
    double max,
    String invalidMin,
    String invalidMax,
  }) {
    if (min != null && value < min) {
      return invalidMin;
    } else if (max != null && value > max) {
      return invalidMax;
    }
    return null;
  }
}
